package ch.usi.inf.demo.jdbc;

import java.sql.Connection;
import java.sql.Date;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;

import org.postgresql.util.PSQLState;

/**
 * 
 * @author reza
 *
 */
public class Demo {

	public static void main(String[] args) {
		
		
		// MySql DB url: jdbc:mysql://localhost:3306/
		try {
			
			/*
			 Before JDBC 4.0
			 Class.forName("org.postgresql.Driver");
			 */
			
			
			//Type 1
			//Properties props = new Properties();
			//props.put("user", value)
			//props.put("password", "")
			//DriverManager.getConnection("jdbc:postgresql://localhost:5432/university", props);
			
			//Type 2
			//DriverManager.getConnection("jdbc:postgresql://localhost:5432/university", "", "");
			
			//Type 3
			Connection conn = DriverManager.getConnection("jdbc:postgresql://localhost:5432/university");
			
			initializeDatabase(conn);
			createTables(conn);
			insertWithStatement(conn);
			insertWithPreparedStmt(conn);
			updateWithStatement(conn);
			selectQuery(conn);
			
			System.out.println("Bye!");
		} catch (SQLException e) {
			e.printStackTrace();
			e.getNextException().printStackTrace();
		}
	}
	
	
	public static void createTables(Connection conn) throws SQLException {
		Statement stmt = conn.createStatement();
		final String studentTableQuery = "create table student ("
				+ "name varchar not null , "
				+ "dob date, "
				+ "gpa integer, "
				+ "s_id integer,"
				+ "primary key (s_id));";
		
		stmt.executeUpdate(studentTableQuery);
		
		final String courseTableQuery = "create table course ("
				+ "c_id integer,"
				+ "credit integer,"
				+ "title varchar(40),"
				+ "primary key (c_id));";
		stmt.executeUpdate(courseTableQuery);
		
		final String hasTakenTableQuery = "create table has_taken ("
				+ "sid integer not null,"
				+ "cid integer not null,"
				+ "year integer not null,"
				+ "primary key (sid, cid),"
				+ "foreign key (cid) references course(c_id),"
				+ "foreign key (sid) references student(s_id)"
				+ ");";
		stmt.executeUpdate(hasTakenTableQuery);
	}
	
	public static void insertWithStatement(Connection conn) throws SQLException{
		Statement stmt = conn.createStatement();
		String s1 = "insert into student values ('s1', null, 9, 1);";
		int affected = stmt.executeUpdate(s1);
		System.out.println("Rows inserted:\t"+affected);
		
		String s2 = "insert into student values ('s2', null, 8, 2);";
		affected = stmt.executeUpdate(s2);
		System.out.println("Rows inserted:\t"+affected);
	}
	
	public static void updateWithStatement(Connection conn) throws SQLException{
		Statement stmt = conn.createStatement();
		String updateQuery = "update student set gpa = 7 where s_id = 1";
		int affected = stmt.executeUpdate(updateQuery);
		System.out.println("Rows updated:\t"+affected);
	}
	
	public static void insertWithPreparedStmt(Connection conn) throws SQLException {
		PreparedStatement pstmt = conn.prepareStatement("insert into student values (?, ?, ?, ?);");
		
		pstmt.setString(1, "s5");
		pstmt.setDate(2, new Date(new java.util.Date().getTime()));
		pstmt.setInt(3, 9);
		pstmt.setInt(4, 5);
		
		int affected = pstmt.executeUpdate();
		System.out.println("Rows inserted:\t"+affected);
	}
	
	public static void selectQuery(Connection conn) throws SQLException{
		Statement stmt = conn.createStatement();
		String retrievalQuery = "select * from student";
		ResultSet rs = stmt.executeQuery(retrievalQuery);
		
		while (rs.next()){
			String name = rs.getString("name");	//using column name
			int gpa = rs.getInt(3);			//using column index starting from 1
			System.out.println("Name:\t"+name+"\tgpa:\t"+gpa);
		}
	}
	
	public static void initializeDatabase(Connection conn) throws SQLException{
		Statement stmt = conn.createStatement();
		String query1 = "drop table if exists has_taken;";
		stmt.addBatch(query1);
		
		String query2 = "drop table if exists student;";
		stmt.addBatch(query2);;
		
		String query3 = "drop table if exists course;";
		stmt.addBatch(query3);
		
		int[] affected = stmt.executeBatch();
		
		int sum = 0;
		for (int i : affected) {
			sum += i;
		}
		
		System.out.println("Number of tables removed: "+sum);
	}
}
